% Fourierova transformace - jednoduchy signal
% signal s(t) = 5 + 2*cos(2*pi*t-90deg) + 3*cos(4*pi*t)
%  signal obsahuje dve frekvence
%  cas t in (0,3/4)
%  uvazuju celkem N=4 vzorky za sekundu => f = 4 Hz => T = 1/4;
%  deleni casove osy je potom [0,1*T,2*T,3*T];
clear all; close all; clc;


%  definuju anonymni funkce
Fsig1 = @(t,degRad) 2*cos(2*pi.*t - degRad);
Fsig2 = @(t) 3*cos(4*pi.*t);

N = 4; % pocet vzorku
f = 4; % vzorkovaci frekvence
T = 1/f; % vzorkovaci perioda
stupne = 90;
degRad = stupne*pi/180;

t = 0:T:(N-1)*T; % casovy vektor

% moje vzorky
s_t = 5 + Fsig1(t,degRad) + Fsig2(t);

%  vytvorim matici koeficientu

koef = zeros(N,N);
W = exp(-i*2*pi/double(N));

for i= 1:N
   for j= 1:N      
       koef(i,j) = W^((i-1)*(j-1));
   end    
end

% vyresim soustavu S_prime = koef*s_t'
S_prime = zeros(1,N); % vektor vyslednych hodnot
for i=1:N        
    soucin = 0;
   for j= 1:N           
    soucin = soucin + koef(i,j)*s_t(j);       
   end
   S_prime(i) = soucin;       
end

S_prime = abs(S_prime);

% vykreslim vysledky
tkresli = 0:0.01:(N-1)*T; % vytvorim novy vektor pro vykresleni 
h = figure;
subplot(2,1,1)
plot(tkresli, Fsig1(tkresli,degRad),'-b'); hold on
plot(tkresli, Fsig2(tkresli),'-r'); hold on
s_tkresli = 5 + Fsig1(tkresli,degRad) + Fsig2(tkresli);
plot(tkresli, s_tkresli,'-g','linewidth',2); 
legend('2*cos(2*pi*t - 90)','3*cos(4*pi*t)','5 + 2*cos(...) + 3*cos(...)');
xlabel('cas t'); ylabel('s(t)');

subplot(2,1,2)
nasobek = 0:1:(N-1);
stem(nasobek,S_prime);
xlabel('ferkvence f'); ylabel('N(f)');
xmax = N-1 + 0.05;
xlim([-0.05 xmax]);

%  porovnani s vestavenou funkci Matlabu

DftMatlab = fft(s_t);
S_primeMatlab= abs(DftMatlab);

h1 = figure;
nasobek = 0:1:(N-1);
stem(nasobek,S_prime,'--r','linewidth',3); hold on;
stem(nasobek,S_primeMatlab,'-b'); hold on;
xlim([-0.05 xmax]);
legend('MojeDFT','Matlab');

%  provedu zpetnou rekonstrukci signalu
sig2Dft = ifft(S_primeMatlab);
%  hodnoty se budou listit, protoze mam malo informaci a velkou chybu
%  rekonstrukce signalu


